%% Recovery - Raster
% Cedric Yue Sik Kin, Andrew Ponec
%
% Computes the albedo and depth map after scene capture through raster
clear; close all;

num_rows = 608;
num_cols = 684;
pixel_size = 20;

res_x = floor(num_cols/pixel_size);
res_y = floor(num_rows/pixel_size);

load('raster_capture_size20.mat');

% average each row to get final values
amplitudes = mean(amplitudes_meas, 2)';
phases = mean(phases_meas, 2)';

db = (amplitudes - 0.881) / 0.030;
mag = 10.^(db / 20);
phase = (phases - 0.996) / 0.0107 * pi / 180;

magnitude = reshape(mag, res_y, res_x)';
phase = reshape(phase, res_y, res_x)';

%% Deconvolution of Phase Data with TV Prior
addpath('toolbox_optim/toolbox_optim');
addpath('toolbox_optim/toolbox_optim/toolbox/');

y = phase;

lambda = .02;

K = @(x)grad(x);
KS = @(x)-div(x);

Amplitude = @(u)sqrt(sum(u.^2,3));
F = @(u)lambda*sum(sum(Amplitude(u)));
G = @(x)1/2*norm(y-x,'fro')^2;

Normalize = @(u)u./repmat( max(Amplitude(u),1e-10), [1 1 2] );
ProxF = @(u,tau)repmat( perform_soft_thresholding(Amplitude(u),lambda*tau), [1 1 2]).*Normalize(u);
ProxFS = compute_dual_prox(ProxF);

ProxG = @(x,tau)(x+tau*y)/(1+tau);

options.niter = 300;
[xAdmm,EAdmm] = perform_admm(y, K,  KS, ProxFS, ProxG, options);

phase_d = xAdmm;

%%

phase_c = phase(:, 4:end);
phase_d_c = phase_d(:, 4:end);
magnitude_c = magnitude(:, 4:end);

tg_min = 0;
tg_max = 1;

curr_min = min(phase_d_c(:));
curr_max = max(phase_d_c(:));
phase_d_r = ((phase_d_c-curr_min)*(tg_max-tg_min))/(curr_max-curr_min) + tg_min;

curr_min = min(phase_c(:));
curr_max = max(phase_c(:));
phase_r = ((phase_c-curr_min)*(tg_max-tg_min))/(curr_max-curr_min) + tg_min;

%% Intensity Map

mag_plot = magnitude_c.*2;
[oldHeight] = size(mag_plot,1);
newWidth = int32(oldHeight * 16/9);

f = figure;
mag_resized = imresize(mag_plot, [oldHeight newWidth]);
imshow(mag_resized, [], 'InitialMagnification','fit');
saveas(f, 'raster_int.png');

%% Assigning Depths to Raw Data

d_square = 1.32;
d_S = 1.57;
d_back = 2.1;

square_ind = [15 19];
back_ind = [30 13];

slope = (d_square - d_back)/(phase_c(square_ind(1),square_ind(2)) ...
    - phase_c(back_ind(1), back_ind(2)));

intercept = d_square - slope * phase_c(square_ind(1),square_ind(2));

depth_map = slope .* phase_c + intercept;
% figure; imagesc(depth_map); colorbar

% Heat Map of Depths - Raw
depth_heatmap = zeros(res_x, res_y - 3, 3);
depth_heatmap(:,:,1) = (1 - phase_r) ./ 2;
depth_heatmap(:,:,2) = 1;
depth_heatmap(:,:,3) = 1;

depth_heatmap = hsv2rgb(depth_heatmap);

[oldHeight] = size(depth_heatmap, 1);
newWidth = int32(oldHeight * 16/9);
depth_heatmap = imresize(depth_heatmap, [oldHeight newWidth]);

f = figure; colormap('hsv'); 
imshow(depth_heatmap, [], 'InitialMagnification','fit'); 
caxis([min(depth_map(:)) max(depth_map(:))]); 
cb = colorbar; ylabel(cb, '( in m )'); set(gca,'fontsize',18, 'FontWeight','Bold')
saveas(f, 'raster_depth_raw.png');

% Heat Map of Depth Combined with Amplitude - Raw
depth_heatmap_c = zeros(res_x, res_y - 3, 3);
depth_heatmap_c(:,:,1) = (1 - phase_r) ./ 2;
depth_heatmap_c(:,:,2) = 1;
depth_heatmap_c(:,:,3) = magnitude_c.*5;

depth_heatmap_c = hsv2rgb(depth_heatmap_c);

[oldHeight] = size(depth_heatmap_c, 1);
newWidth = int32(oldHeight * 16/9);
depth_heatmap_c = imresize(depth_heatmap_c, [oldHeight newWidth]);

f = figure; colormap('hsv'); 
imshow(depth_heatmap_c, [], 'InitialMagnification','fit'); 
caxis([min(depth_map(:)) max(depth_map(:))]); 
cb = colorbar; ylabel(cb, '( in m )'); set(gca,'fontsize',18, 'FontWeight','Bold')
saveas(f, 'raster_comb_raw.png');



%% Assigning Depths to Denoised Data

slope = (d_square - d_back)/(phase_d_c(square_ind(1),square_ind(2)) ...
    - phase_d_c(back_ind(1), back_ind(2)));

intercept = d_square - slope * phase_d_c(square_ind(1),square_ind(2));

depth_map_d = slope .* phase_d_c + intercept;
% figure; imagesc(depth_map); colorbar

% Heat Map of Depths - Denoised
depth_heatmap_d = zeros(res_x, res_y - 3, 3);
depth_heatmap_d(:,:,1) = (1 - phase_d_r) ./ 2;
depth_heatmap_d(:,:,2) = 1;
depth_heatmap_d(:,:,3) = 1;

depth_heatmap_d = hsv2rgb(depth_heatmap_d);

[oldHeight] = size(depth_heatmap_d, 1);
newWidth = int32(oldHeight * 16/9);
depth_heatmap_d = imresize(depth_heatmap_d, [oldHeight newWidth]);

f = figure; colormap('hsv'); 
imshow(depth_heatmap_d, [], 'InitialMagnification','fit'); 
caxis([min(depth_map_d(:)) max(depth_map_d(:))]); 
cb = colorbar; ylabel(cb, '( in m )'); set(gca,'fontsize',18, 'FontWeight','Bold')
saveas(f, 'raster_depth_den.png');

% Heat Map of Depths Combined With Intensity - Denoised
depth_heatmap_d_c = zeros(res_x, res_y - 3, 3);
depth_heatmap_d_c(:,:,1) = (1 - phase_d_r) ./ 2;
depth_heatmap_d_c(:,:,2) = 1;
depth_heatmap_d_c(:,:,3) = magnitude_c.*5;

depth_heatmap_d_c = hsv2rgb(depth_heatmap_d_c);

[oldHeight] = size(depth_heatmap_d_c, 1);
newWidth = int32(oldHeight * 16/9);
depth_heatmap_d_c = imresize(depth_heatmap_d_c, [oldHeight newWidth]);

f = figure; colormap('hsv'); 
imshow(depth_heatmap_d_c, [], 'InitialMagnification','fit'); 
caxis([min(depth_map_d(:)) max(depth_map_d(:))]); 
cb = colorbar; ylabel(cb, '( in m )'); set(gca,'fontsize',18, 'FontWeight','Bold')
saveas(f, 'raster_comb_den.png');